use std::{collections::HashMap, net::SocketAddr, sync::Arc};

use serde::{Deserialize, Serialize};
use tokio::{net::UdpSocket, sync::{mpsc, Mutex}, task};

#[derive(Serialize, Deserialize, Debug)]
enum DHTMessage {
    Put { key: String, value: String },
    Get { key: String },
    Response { value: Option<String> },
}

async fn server() {
    let addr = "127.0.0.1:8080"; // Change this per node

    let socket = UdpSocket::bind(addr).await.unwrap();

    let storage = Arc::new(Mutex::new(HashMap::new()));

    let (tx, mut rx) = mpsc::channel::<(SocketAddr, DHTMessage)>(100);

    let storage_clone = storage.clone();

    task::spawn(async move {
        while let Some((peer, msg)) = rx.recv().await {
            let response = match msg {
                DHTMessage::Put { key, value } => {
                    storage_clone.lock().await.insert(key.clone(), value.clone());
                    DHTMessage::Response { value: Some("OK".to_string()) }
                }
                DHTMessage::Get { key } => {
                    let value = storage_clone.lock().await.get(&key).cloned();
                    DHTMessage::Response { value }
                }
                _ => DHTMessage::Response { value: None },

            };
            
            let response_bytes = bincode::serialize(&response).unwrap();
            socket.send_to(&response_bytes, peer).await.unwrap();
        }
    });

    /*let mut buf = [0; 1024];

    loop {
        let (size, peer) = socket.recv_from(&mut buf).await.unwrap();

        if let Ok(msg) = bincode::deserialize::<DHTMessage>(&buf[..size]) {
            tx.send((peer, msg)).await.unwrap();
        }
    }*/
}